@@ -2,7 +2,7 @@ import json
 import time
 from collections import namedtuple
 from jsonpath_ng.ext import parse
-from ixnetwork_restpy import SessionAssistant
+from ixnetwork_restpy import SessionAssistant, StatViewAssistant
 from abstract_open_traffic_generator.api import *
 from abstract_open_traffic_generator.config import *
 from abstract_open_traffic_generator.control import *
@@ -28,7 +28,8 @@ class IxNetworkApi(Api):
                  address='127.0.0.1',
                  port='11009',
                  username='admin',
-                 password='admin'):
+                 password='admin',
+                 license_servers=[]):
         """Create a session
         - address (str): The ip address of the TestPlatform to connect to 
         where test sessions will be created or connected to.
@@ -41,6 +42,7 @@ class IxNetworkApi(Api):
         self._port = port
         self._username = username
         self._password = password
+        self._license_servers = license_servers
         self._running_config = None
         self._config = None
         self._assistant = None
@@ -63,11 +65,16 @@ class IxNetworkApi(Api):
         return self._ixn_objects
 
     def get_ixn_object(self, name):
-        """Returns a ixnetwork_restpy object given a unique configuration name
+        """Returns an ixnetwork_restpy object given a unique configuration name
         """
-        href = self._ixn_objects[name]
+        href = self.get_ixn_href(name)
         return self._assistant.Session.GetObjectFromHref(href)
 
+    def get_ixn_href(self, name):
+        """Returns an href given a unique configuration name
+        """
+        return self._ixn_objects[name]
+
     @property
     def assistant(self):
         return self._assistant
@@ -132,11 +139,13 @@ class IxNetworkApi(Api):
     def _set_flow_transmit_state(self, flow_transmit_state):
         """Set the transmit state of flows
         """
+        self._connect()
         return self.traffic_item.transmit(flow_transmit_state)
 
     def _set_port_capture_state(self, request):
         """Starts capture on all ports that have capture enabled.
         """
+        self._connect()
         self._capture_request = request
 
     def _start_capture(self):
@@ -174,6 +183,7 @@ class IxNetworkApi(Api):
     def get_port_results(self, request):
         """Abstract API implementation
         """
+        self._connect()
         return self.vport.results(request)
 
     def get_flow_results(self, request):
@@ -220,11 +230,13 @@ class IxNetworkApi(Api):
                 UserName=self._username,
                 Password=self._password,
                 LogLevel=SessionAssistant.LOGLEVEL_INFO)
-            self._ixnetwork = self._assistant.Ixnetwork
+            self._ixnetwork = self._assistant.Session.Ixnetwork
             self._vport = self._ixnetwork.Vport
             self._topology = self._ixnetwork.Topology
             self._traffic = self._ixnetwork.Traffic
             self._traffic_item = self._ixnetwork.Traffic.TrafficItem
+            if len(self._license_servers) > 0:
+                self._ixnetwork.Globals.Licensing.LicensingServers = self._license_servers
 
     def _request(self, method, url, payload=None):
         connection, url = self._assistant.Session._connection._normalize_url(
@@ -410,34 +422,25 @@ class IxNetworkApi(Api):
         results = self._ixnetwork._connection._execute(url, payload)
         return results[0]['chassis'][0]['card'][0]['port'][0]['xpath']
 
-    def clear_ownership(self, hrefs):
-        if len(hrefs) > 0:
-            self.info('Pre-empting locations [%s]' %
-                                 ', '.join([href for href in hrefs.keys()]))
-            url = '%s/operations/clearownership' % [
-                href for href in hrefs.values()
-            ][0]
-            payload = {'arg1': [href for href in hrefs.values()]}
-            results = self._ixnetwork._connection._execute(url, payload)
-            time.sleep(2)
+    def clear_ownership(self, available_hardware_hrefs, location_hrefs):
+        for ownership_hrefs in [available_hardware_hrefs, location_hrefs]:
+            if len(ownership_hrefs) > 0:
+                payload = {'arg1': [href for href in ownership_hrefs.values()]}
+                self.info('Pre-empting locations [%s]' %
+                                    ', '.join([location for location in ownership_hrefs.keys()]))
+                url = '%s/operations/clearownership' % payload['arg1'][0]
+                results = self._ixnetwork._connection._execute(url, payload)
 
     def get_config(self):
         return self._config
 
-    def protocol_statistics(self):
-        # from ixnetwork_restpy import StatViewAssistant
-        # view = StatViewAssistant(self._ixnetwork,
-        #                          'Protocols Summary',
-        #                          Timeout=10,
-        #                          PrintColumns=[
-        #                              'Protocol Name', 'Sessions Up',
-        #                              'Sessions Down', 'Sessions Total'
-        #                          ],
-        #                          PrintFormat=StatViewAssistant.TABLE)
-        # for topology in self._topology.find():
-        #     self._ixnetwork.info('%s protocols total:%s started:%s' % (topology.stat))
-        # self._ixnetwork.info(view)
-        pass
+    def check_protocol_statistics(self):
+        view = StatViewAssistant(self._ixnetwork, 'Protocols Summary')
+        view.CheckCondition('Sessions Not Started', StatViewAssistant.EQUAL, 0)
+        view.CheckCondition('Sessions Down', StatViewAssistant.EQUAL, 0)
 
-    def info(self, info):
-        self._ixnetwork.info('[ixn-otg] %s' % info)
+    def info(self, message):
+        self._ixnetwork.info('[ixn-otg] %s' % message)
+
+    def warning(self, message):
+        self._ixnetwork.warn('[ixn-otg] %s' % message)
