@@ -160,30 +160,34 @@ class Vport(object):
 
     def _add_hosts(self, HostReadyTimeout):
         chassis = self._api._ixnetwork.AvailableHardware.Chassis
-        ip_addresses = []
+        add_addresses = []
+        check_addresses = []
         for port in self._api.config.ports:
             if port.location is not None and ';' in port.location:
                 chassis_address = port.location.split(';')[0]
                 chassis.find(Hostname='^(%s)$' % chassis_address)
                 if len(chassis) == 0:
-                    ip_addresses.append(chassis_address)
-        ip_addresses = set(ip_addresses)
-        if len(ip_addresses) == 0:
-            return
-        self._api.info('Adding location hosts [%s]...' %
-                                  ', '.join(ip_addresses))
-        for ip_address in ip_addresses:
-            chassis.add(Hostname=ip_address)
+                    add_addresses.append(chassis_address)
+                check_addresses.append(chassis_address)
+        add_addresses = set(add_addresses)
+        check_addresses = set(check_addresses)
+        if len(add_addresses) > 0:
+            self._api.info('Adding location hosts [%s]...' %
+                                    ', '.join(add_addresses))
+            for add_address in add_addresses:
+                chassis.add(Hostname=add_address)
         start_time = time.time()
+        self._api.info('Checking state of location hosts [%s]...' %
+                                ', '.join(check_addresses))
         while True:
-            chassis.find(Hostname='^(%s)$' % '|'.join(ip_addresses),
+            chassis.find(Hostname='^(%s)$' % '|'.join(check_addresses),
                          State='^ready$')
-            if len(chassis) == len(ip_addresses):
+            if len(chassis) == len(check_addresses):
                 break
             if time.time() - start_time > HostReadyTimeout:
                 raise RuntimeError(
                     'After %s seconds, not all location hosts [%s] are reachable'
-                    % (HostReadyTimeout, ', '.join(ip_addresses)))
+                    % (HostReadyTimeout, ', '.join(check_addresses)))
             time.sleep(2)
 
     def _set_location(self):
@@ -223,9 +227,11 @@ class Vport(object):
             if location is not None and len(location) > 0:
                 clear_locations.append(location)
                 locations.append(port.name)
+        if len(locations) == 0:
+            return
         self._clear_ownership(clear_locations)
         self._api.info('Connecting locations [%s]...' %
-                                  ', '.join(locations))
+                                ', '.join(locations))
         self._import(imports)
         self._api.info('Checking location state [%s]...' %
                                   ', '.join(locations))
@@ -242,8 +248,10 @@ class Vport(object):
             if time.time() - start > timeout:
                 raise RuntimeError(
                     'After %s seconds, not all locations [%s] are reachable' %
-                    (timeout, ', '.join(locations)))
+                    (timeout, ', '.join([vport.Name for vport in self._api._vport])))
             time.sleep(2)
+        for vport in self._api._vport.find(ConnectionState='^(?!connectedLinkUp).*$'):
+            self._api.warning('%s %s' % (vport.Name, vport.ConnectionState))
 
     def _set_layer1(self):
         """Set the /vport/l1Config/... properties
@@ -302,11 +310,12 @@ class Vport(object):
             mode = speed_mode_map[layer1.speed]
             card = self._api.select_chassis_card(vport)
             for available_mode in card['availableModes']:
-                if re.match(mode, available_mode.lower()) is not None:
+                if re.search(mode, available_mode.lower()) is not None:
                     aggregation_mode = available_mode
                     break
         if aggregation_mode is not None and aggregation_mode != card[
                 'aggregationMode']:
+            self._api.info('Setting %s layer1 mode' % aggregation_mode)
             imports.append({
                 'xpath': card['xpath'],
                 'aggregationMode': aggregation_mode
@@ -393,12 +402,9 @@ class Vport(object):
             layer1.ieee_media_defaults,
             'speed':
             Vport._SPEED_MAP[layer1.speed],
-            'enableAutoNegotiation':
-            layer1.auto_negotiate,
-            'enableRsFec':
-            layer1.auto_negotiation.rs_fec,
-            'linkTraining':
-            layer1.auto_negotiation.link_training,
+            'enableAutoNegotiation': layer1.auto_negotiate,
+            'enableRsFec': None if layer1.auto_negotiation is None else layer1.auto_negotiation.rs_fec,
+            'linkTraining': None if layer1.auto_negotiation is None else layer1.auto_negotiation.link_training
         }
         self._add_l1config_import(vport, proposed_import, imports)
 
@@ -448,15 +454,25 @@ class Vport(object):
         except:
             force_ownership = False
         if force_ownership is True:
-            hrefs = {}
-            chassis = self._api._ixnetwork.AvailableHardware.Chassis
+            available_hardware_hrefs = {}
+            location_hrefs = {}
             for location in locations:
-                clp = location.split(';')
-                chassis.find(Hostname=clp[0])
-                if len(chassis) == 1:
-                    hrefs[location] = '%s/card/%s/port/%s' % (
-                        chassis.href, abs(int(clp[1])), abs(int(clp[2])))
-            self._api.clear_ownership(hrefs)
+                if ';' in location:
+                    clp = location.split(';')
+                    chassis = self._api._ixnetwork.AvailableHardware.Chassis.find(Hostname=clp[0])
+                    if len(chassis) > 0:
+                        available_hardware_hrefs[location] = '%s/card/%s/port/%s' % (
+                            chassis.href, abs(int(clp[1])), abs(int(clp[2])))
+                elif '/' in location:
+                    appliance = location.split('/')[0]
+                    locations = self._api._ixnetwork.Locations
+                    locations.find(Hostname=appliance)  
+                    if len(locations) == 0:    
+                        locations.add(Hostname=appliance)
+                    ports = locations.Ports.find(Location='^%s$' % location)
+                    if len(ports) > 0:
+                        location_hrefs[location] = ports.href
+            self._api.clear_ownership(available_hardware_hrefs, location_hrefs)
 
     def _set_result_value(self,
                           row,
